'use strict';

const _                      = require('lodash');
const Promise                = require('bluebird');
const YAML                   = require('js-yaml');
const crypto                 = require('crypto');
const ResourceController     = require('./../resource.base.controller');
const Secret                 = require('./secret');
const ImagePullSecretBuilder = require('./image.pull.secret.builder');
const SecretEvents           = require('./secret.events');
const secretLabels           = require('./secret.labels');



class KubeSecretController extends ResourceController {

    constructor(data) {
        super(data);
    }

    createImagePullSecret({ usersRegistry, credentials, domain, ns, name, labels }) {

        labels                 = _.merge(labels, {
            [secretLabels.IMAGE_PULL_SECRET_HASH.key]: this.buildImagePullSecretHash({
                username: credentials.username,
                password: credentials.password
            }),
            [secretLabels.IMAGE_PULL_SECRET_REGISTRY_DOMAIN.key]: usersRegistry.domain,
            [secretLabels.IMAGE_PULL_SECRET_REGISTRY_PROVIDER.key]: usersRegistry.provider,
            [secretLabels.IMAGE_PULL_SECRET_REGISTRY_ID.key]: usersRegistry._id,
            [secretLabels.IMAGE_PULL_SECRET_REGISTRY_NAME.key]: usersRegistry.name,
        });
        const secretDescriptor = {
            name,
            credentials,
            domain,
            labels
        };

        //TODO use the resouceController create function
        const deferred    = Promise.defer();
        const isYamlBased = !!_.get(secretDescriptor, 'yaml', false);
        let secretTemplate;
        if (isYamlBased) {
            secretTemplate = YAML.load(secretDescriptor.yaml);
        } else {
            secretTemplate = new ImagePullSecretBuilder(secretDescriptor).build();
        }
        this.getKubeClient()
            .namespaces
            .secrets
            .post({ body: secretTemplate }, (err, secret) => {
                if (err) {
                    return deferred.reject(err);
                }
                const sec = new Secret(secret, { selector: this.getSelector() });

                this.publishEvent(SecretEvents.types.ADDED, {
                    name: sec.getName()
                });
                return deferred.resolve(sec);
            });

        return deferred.promise;
    }

    buildImagePullSecretName({ domain, name, ns }) {
        return `codefresh-generated-${domain}-${name}-${ns}`;
    }

    buildImagePullSecretHash({ username, password }) {
        return crypto.createHash(
            'sha256')
            .update(`${username}:${password}`)
            .digest('hex').substring(0, 62)
    }


}

module.exports = KubeSecretController;